#!/usr/bin/env python

import spss, spssaux, re, os, random

__author__  =  'Raynald Levesque raynald@spsstools.net'
__version__ =  '1.0'

# No sample data files are included with this script
# User needs to supply data files, adapt the _getFile() function to the
# data, and use applicable parameters in the loadAllFiles() function 

def getFileList(drivepath="c:/temp/",
                searchSubFolder=False,
                pattern=r"^csv\d+\.csv",
                addFlag = True,
                runSyntax = False ,
                newSyntaxName = r'c:\temp\combined data.sps',
                labelSource = False):
    """getFileList(drivepath="c:/temp/", searchSubFolder=False, filenames=r"file*.sav")
    This function return the list of files whose names meets the given pattern

    It also returns listSavFiles which is the list of filespecs of files meeting pattern
    """

    # protect active dataset
    dsn = spssaux.GetActiveDatasetName()
    if not dsn:
        spss.Submit("dataset name S" + str(random.uniform(0,1)) + " window=asis.")
    spss.Submit("SET MPRINT=YES /PRINTBACK=YES")

    listFiles = []
    pat = re.compile(pattern, re.IGNORECASE)
    msg='folder'
    if searchSubFolder: msg = 'directory tree'
    print "searching %(msg)s %(drivepath)s and filenames like %(pattern)s" % locals()

    f = os.walk(drivepath)  #create generator function
    for (dirname, subdirs, files) in f:
        if dirname == drivepath or searchSubFolder == True:
            for file in files:
                if pat.match(file):
                    if dirname == drivepath:
                        fullname = dirname + file
                    else:
                        fullname = dirname + '/' + file
                    try:
                        _getFile(fullname)
                        savName = fullname[:-3] + 'sav'
                        listFiles.append(savName)
                        filenb = len(listFiles)
                        if labelSource:
                            cmd2 = r"""
                            COMPUTE source=%u.
                            SAVE OUTFILE='%s.'
                            """ % (filenb, savName)
                        else:
                            cmd2 = r"""
                            SAVE OUTFILE='%s.'
                            """ % savName
                        spss.Submit(cmd2)
                        print 'Converted:',fullname,'to',savName
                    except:
                        print '>>>Problem reading file',fullname
                        pass
    return listFiles

def _getFile(filespecs):
    """Replace the content of the next cmd variable by the code required
    to read one of your data file. Make sure to use 'pct s' instead of the
    file path and file name
    """
    cmd = r"""
    GET DATA  /TYPE = TXT
        /FILE = '%s'
        /DELCASE = LINE
        /DELIMITERS = ","
        /QUALIFIER = '"'
        /ARRANGEMENT = DELIMITED
        /FIRSTCASE = 2
        /IMPORTCASE = ALL
        /VARIABLES =
        ID F1.0
        Name A15
        Gender A1
        DateHired ADATE10
        Department F1.0 .
       CACHE.
    """ % filespecs
    try:
        spss.Submit(cmd)
        return 0
    except:
        print '\n>>>This cmd produced an error:\n',cmd
        raise
        return 1

def addFiles(listSavFiles, newFileName, labelSource=False, cmd=''):
    """addFiles(listSavFiles, newFileName, cmd) adds many sav files
    listSavFiles: a list of complete filepecs (i.e. paths and names) of sav files to be added
    newFileName:  the filespec of the new file eg 'c:\data\combined data.sav'
    labelSource: when True a variable in the combined datafile shows the name of
        the source file for each case
    cmd is the current syntax (command string)
    """
    first = True
    listF = listSavFiles[:]
    if len(listSavFiles) > 1:
        while listF:
            if first:
                cmd = cmd + "\n\n* Add Files.\nADD FILES " + ' '.join(["\n /FILE='%s'" % (fn) for fn in listF[:50]]) + ".\n"
                listF = listF[50:]
                first = False
            else:
                cmd = cmd + "EXECUTE.\nADD FILES /FILE=*" + ' '.join(["\n /FILE='%s'" % (fn) for fn in listF[:49]]) + ".\n"
                listF = listF[49:]

        if labelSource:
            for idx in range(len(listSavFiles)):
                if idx > 0:
                    cmd = cmd + '   %u "%s"\n' % (idx+1,listSavFiles[idx])
                else:
                    cmd = cmd + 'ADD VAL LAB source %u "%s"\n' % (idx+1,listSavFiles[idx])
            cmd = cmd.rstrip()+ '.'
        cmd = cmd + '\nCACHE.\nSAVE OUTFILE="%s".' % (newFileName)
    else:
        cmd = cmd + '*There are no files to ADD.\n'
    return cmd

def loadAllFiles(drivepath="c:/temp/",
                searchSubFolder=False,
                pattern=r"^csv\d+\.csv",
                addFlag = True,
                runSyntax = False ,
                newFileName = r'c:/temp/combined data.sav',
                newSyntaxName = r'c:/temp/combined data.sps',
                labelSource = False):

    listFiles = getFileList(drivepath, searchSubFolder, pattern,
                   addFlag, runSyntax, newSyntaxName, labelSource)

    cmd = addFiles(listFiles, newFileName, labelSource)
    print cmd
    spss.Submit(cmd)

if __name__ == '__main__':
    #At least one csv files must exist in c:/temp/
    loadAllFiles(drivepath = "c:/temp/",
                searchSubFolder = True,
                pattern = r"^csv\d+\.csv",
                addFlag = True,
                runSyntax = False ,
                newFileName = r'c:/temp/all csv files.sav',
                newSyntaxName = r'c:/temp/combined data.sps',
                labelSource = True)
